#include <CGAL/basic.h>
#include <CGAL/Simple_cartesian.h>
#include "polyhedron.h"
#include <fstream>
#include <CGAL/IO/Polyhedron_iostream.h>


typedef float number_type;
typedef CGAL::Simple_cartesian<number_type> Enriched_kernel;


int main(int argc, char* argv[])
{
  if(argc <= 1)
  {
    std::cout << "usage: ./genus data/mesh.off" << std::endl;
    return 0;
  }
  // read mesh (off format)
  typedef Enriched_polyhedron<Enriched_kernel,Enriched_items> Mesh;
  Mesh mesh;
  std::cout << "read mesh...";
  std::ifstream input_stream(argv[1]);
  input_stream >> mesh;
  std::cout << "ok" << std::endl;

  // number of primitives
  int v = mesh.size_of_vertices();
  int f = mesh.size_of_facets();
  int e = mesh.size_of_halfedges() / 2;
  std::cout << v << " vertices" << std::endl;
  std::cout << f << " facet(s)"   << std::endl;
  std::cout << e << " edges"    << std::endl;

  // compute genus
  int c = mesh.nb_components();
  int b = mesh.nb_boundaries();
  int g = mesh.genus(c,v,f,e,b);
  std::cout << c << " connected component(s)" << std::endl;
  std::cout << b << " boundary(ies)"   << std::endl;
  std::cout << "genus " << g << std::endl;

  return 1;
}
  

  
  
