#ifndef	_POLYGON_MESH_
#define	_POLYGON_MESH_

// CGAL	stuff
#include <CGAL/Cartesian.h>
#include <CGAL/Polyhedron_3.h>
#include <list>

// a refined facet with a tag
template <class	Refs,	class	T, class P,	class	Norm>
class	Enriched_facet : public	CGAL::HalfedgeDS_face_base<Refs, T>
{
 // tag
 int m_tag; 

public:

 // life cycle
 // no constructors to repeat, since only
 // default constructor mandatory

 Enriched_facet()
 {
 }

 // tag
 const int& tag() { return m_tag; }
 void tag(const int& t) { m_tag = t; }

};

// a refined halfedge with a general tag
template <class Refs, class Tprev, class Tvertex, class Tface, class Norm>
class Enriched_halfedge : public CGAL::HalfedgeDS_halfedge_base<Refs,Tprev,Tvertex,Tface>
{
private:

 // tag
 int m_tag; 

public:

 // life cycle
 Enriched_halfedge()
 {
 }

 // tag
 const int& tag() const { return m_tag; }
 int& tag() { return m_tag; }
 void tag(const int& t) { m_tag = t; }

};



// a refined vertex with a tag
template <class Refs, class T, class P, class Norm>
class Enriched_vertex : public CGAL::HalfedgeDS_vertex_base<Refs, T, P>
{
 // tag
 int m_tag; 

public:
 // life cycle
 Enriched_vertex()  {}
 // repeat mandatory constructors
 Enriched_vertex(const P& pt)
  : CGAL::HalfedgeDS_vertex_base<Refs, T, P>(pt)
 {
 }

 // tag
 int& tag() { return m_tag; }
 const int& tag() const { return m_tag; }
 void tag(const int& t) { m_tag = t; }
};

// A redefined items class for the Polyhedron_3 
// with a refined vertex class that contains a 
// member for the normal vector and a refined
// facet with a normal vector instead of the 
// plane equation (this is an alternative 
// solution instead of using 
// Polyhedron_traits_with_normals_3).

struct Enriched_items : public CGAL::Polyhedron_items_3
{
  // wrap vertex
  template <class Refs, class Traits>
  struct Vertex_wrapper
  {
    typedef typename Traits::Point_3 Point;
    typedef typename Traits::Vector_3 Normal;
    typedef Enriched_vertex<Refs,
             CGAL::Tag_true,
             Point,
             Normal> Vertex;
  };

  // wrap face
  template <class Refs, class Traits>
  struct Face_wrapper
  {
    typedef typename Traits::Point_3 Point;
    typedef typename Traits::Vector_3 Normal;
    typedef Enriched_facet<Refs,
             CGAL::Tag_true,
             Point,
             Normal> Face;
  };

  // wrap halfedge
  template <class Refs, class Traits>
  struct Halfedge_wrapper
  {
    typedef typename Traits::Vector_3 Normal;
    typedef Enriched_halfedge<Refs,
              CGAL::Tag_true,
              CGAL::Tag_true,
              CGAL::Tag_true,
              Normal> Halfedge;
  };
};

//*********************************************************
template <class kernel, class items>
class Enriched_polyhedron : public CGAL::Polyhedron_3<kernel,items>
{
public :
  typedef typename kernel::FT FT;
  typedef typename kernel::Point_3 Point;
  typedef typename kernel::Vector_3 Vector;
  typedef typename CGAL::Polyhedron_3<kernel,items> base;
  typedef typename base::Facet_handle Facet_handle;
  typedef typename base::Vertex_handle Vertex_handle;
  typedef typename base::Halfedge_handle Halfedge_handle;
  typedef typename base::Facet_iterator Facet_iterator;
  typedef typename base::Halfedge_iterator Halfedge_iterator;
  typedef typename base::Vertex_iterator Vertex_iterator;
  typedef typename base::Halfedge_around_facet_circulator
  Halfedge_around_facet_circulator;



private :

public :

  // life cycle
  Enriched_polyhedron() 
    {
    }
  virtual ~Enriched_polyhedron() 
    {
    }

  // tag all halfedges
  void tag_halfedges(const int tag)
    {
      for(Halfedge_iterator pHalfedge = this->halfedges_begin();
	  pHalfedge != this->halfedges_end();
	  pHalfedge++)
	pHalfedge->tag(tag);
    }
  
  // tag all facets
  void tag_facets(const int tag)
    {
      for(Facet_iterator pFace = this->facets_begin();
	  pFace != this->facets_end();
	  pFace++)
	pFace->tag(tag);
    }
  
  // count #boundaries
  // return the number of boundary backbones
  int nb_boundaries()
    {
      int nb = 0;
      // todo
      return nb;
    }
  
  // count #components
  int nb_components()
    {
      int nb = 0;
      // todo 
      return nb;
    }
  
  // compute the genus
  // V - E + F + B = 2 (C - G)
  // C -> #connected components
  // G : genus
  // B : #boundaries
  int genus(int	c,
	    int v,
	    int	f,
	    int e,
	    int b)
    {
      // todo
      return 0;
    }
};


#endif // _POLYGON_MESH_

